@@ -10,6 +10,7 @@ import asyncio
 from typing import Any
 from typing import Awaitable
 from typing import Callable
+from typing import cast
 from typing import Dict
 from typing import Generic
 from typing import Iterable
@@ -570,7 +571,6 @@ class AsyncSession(ReversibleProxy[Session]):
         identity_token: Optional[Any] = None,
         execution_options: OrmExecuteOptionsParameter = util.EMPTY_DICT,
     ) -> Optional[_O]:
-
         """Return an instance based on the given primary key identifier,
         or ``None`` if not found.
 
@@ -581,8 +581,10 @@ class AsyncSession(ReversibleProxy[Session]):
 
         """
 
+        # result_obj = self.sync_session.get(entity, ident)
+
         result_obj = await greenlet_spawn(
-            self.sync_session.get,
+            cast("Callable[..., _O]", self.sync_session.get),
             entity,
             ident,
             options=options,
@@ -625,7 +627,6 @@ class AsyncSession(ReversibleProxy[Session]):
         bind_arguments: Optional[_BindArguments] = None,
         **kw: Any,
     ) -> AsyncResult[Any]:
-
         """Execute a statement and return a streaming
         :class:`_asyncio.AsyncResult` object.
 
