@@ -5,6 +5,8 @@
 # This module is part of SQLAlchemy and is released under
 # the MIT License: https://www.opensource.org/licenses/mit-license.php
 
+from __future__ import annotations
+
 from typing import Optional
 from typing import Sequence
 
@@ -14,6 +16,7 @@ from mypy.nodes import AssignmentStmt
 from mypy.nodes import CallExpr
 from mypy.nodes import Expression
 from mypy.nodes import FuncDef
+from mypy.nodes import LambdaExpr
 from mypy.nodes import MemberExpr
 from mypy.nodes import NameExpr
 from mypy.nodes import RefExpr
@@ -35,18 +38,20 @@ from . import names
 from . import util
 
 
-def _infer_type_from_right_hand_nameexpr(
+def infer_type_from_right_hand_nameexpr(
     api: SemanticAnalyzerPluginInterface,
     stmt: AssignmentStmt,
     node: Var,
     left_hand_explicit_type: Optional[ProperType],
-    infer_from_right_side: NameExpr,
+    infer_from_right_side: RefExpr,
 ) -> Optional[ProperType]:
-
-    type_id = names._type_id_for_callee(infer_from_right_side)
-
+    type_id = names.type_id_for_callee(infer_from_right_side)
     if type_id is None:
         return None
+    elif type_id is names.MAPPED:
+        python_type_for_type = _infer_type_from_mapped(
+            api, stmt, node, left_hand_explicit_type, infer_from_right_side
+        )
     elif type_id is names.COLUMN:
         python_type_for_type = _infer_type_from_decl_column(
             api, stmt, node, left_hand_explicit_type
@@ -60,7 +65,7 @@ def _infer_type_from_right_hand_nameexpr(
             api, stmt, node, left_hand_explicit_type
         )
     elif type_id is names.SYNONYM_PROPERTY:
-        python_type_for_type = _infer_type_from_left_hand_type_only(
+        python_type_for_type = infer_type_from_left_hand_type_only(
             api, node, left_hand_explicit_type
         )
     elif type_id is names.COMPOSITE_PROPERTY:
@@ -128,8 +133,8 @@ def _infer_type_from_relationship(
     # string expression
     # isinstance(target_cls_arg, StrExpr)
 
-    uselist_arg = util._get_callexpr_kwarg(stmt.rvalue, "uselist")
-    collection_cls_arg: Optional[Expression] = util._get_callexpr_kwarg(
+    uselist_arg = util.get_callexpr_kwarg(stmt.rvalue, "uselist")
+    collection_cls_arg: Optional[Expression] = util.get_callexpr_kwarg(
         stmt.rvalue, "collection_class"
     )
     type_is_a_collection = False
@@ -137,7 +142,7 @@ def _infer_type_from_relationship(
     # this can be used to determine Optional for a many-to-one
     # in the same way nullable=False could be used, if we start supporting
     # that.
-    # innerjoin_arg = _get_callexpr_kwarg(stmt.rvalue, "innerjoin")
+    # innerjoin_arg = util.get_callexpr_kwarg(stmt.rvalue, "innerjoin")
 
     if (
         uselist_arg is not None
@@ -147,7 +152,7 @@ def _infer_type_from_relationship(
         type_is_a_collection = True
         if python_type_for_type is not None:
             python_type_for_type = api.named_type(
-                "__builtins__.list", [python_type_for_type]
+                names.NAMED_TYPE_BUILTINS_LIST, [python_type_for_type]
             )
     elif (
         uselist_arg is None or api.parse_bool(uselist_arg) is True
@@ -218,7 +223,7 @@ def _infer_type_from_relationship(
             util.fail(api, msg.format(node.name), node)
 
     if python_type_for_type is None:
-        return _infer_type_from_left_hand_type_only(
+        return infer_type_from_left_hand_type_only(
             api, node, left_hand_explicit_type
         )
     elif left_hand_explicit_type is not None:
@@ -245,7 +250,7 @@ def _infer_type_from_decl_composite_property(
     node: Var,
     left_hand_explicit_type: Optional[ProperType],
 ) -> Optional[ProperType]:
-    """Infer the type of mapping from a CompositeProperty."""
+    """Infer the type of mapping from a Composite."""
 
     assert isinstance(stmt.rvalue, CallExpr)
     target_cls_arg = stmt.rvalue.args[0]
@@ -260,7 +265,7 @@ def _infer_type_from_decl_composite_property(
         python_type_for_type = None
 
     if python_type_for_type is None:
-        return _infer_type_from_left_hand_type_only(
+        return infer_type_from_left_hand_type_only(
             api, node, left_hand_explicit_type
         )
     elif left_hand_explicit_type is not None:
@@ -271,6 +276,38 @@ def _infer_type_from_decl_composite_property(
         return python_type_for_type
 
 
+def _infer_type_from_mapped(
+    api: SemanticAnalyzerPluginInterface,
+    stmt: AssignmentStmt,
+    node: Var,
+    left_hand_explicit_type: Optional[ProperType],
+    infer_from_right_side: RefExpr,
+) -> Optional[ProperType]:
+    """Infer the type of mapping from a right side expression
+    that returns Mapped.
+
+
+    """
+    assert isinstance(stmt.rvalue, CallExpr)
+
+    # (Pdb) print(stmt.rvalue.callee)
+    # NameExpr(query_expression [sqlalchemy.orm._orm_constructors.query_expression])  # noqa: E501
+    # (Pdb) stmt.rvalue.callee.node
+    # <mypy.nodes.FuncDef object at 0x7f8d92fb5940>
+    # (Pdb) stmt.rvalue.callee.node.type
+    # def [_T] (default_expr: sqlalchemy.sql.elements.ColumnElement[_T`-1] =) -> sqlalchemy.orm.base.Mapped[_T`-1]  # noqa: E501
+    # sqlalchemy.orm.base.Mapped[_T`-1]
+    # the_mapped_type = stmt.rvalue.callee.node.type.ret_type
+
+    # TODO: look at generic ref and either use that,
+    # or reconcile w/ what's present, etc.
+    the_mapped_type = util.type_for_callee(infer_from_right_side)  # noqa
+
+    return infer_type_from_left_hand_type_only(
+        api, node, left_hand_explicit_type
+    )
+
+
 def _infer_type_from_decl_column_property(
     api: SemanticAnalyzerPluginInterface,
     stmt: AssignmentStmt,
@@ -284,23 +321,38 @@ def _infer_type_from_decl_column_property(
 
     """
     assert isinstance(stmt.rvalue, CallExpr)
-    first_prop_arg = stmt.rvalue.args[0]
 
-    if isinstance(first_prop_arg, CallExpr):
-        type_id = names._type_id_for_callee(first_prop_arg.callee)
+    if stmt.rvalue.args:
+        first_prop_arg = stmt.rvalue.args[0]
 
-        # look for column_property() / deferred() etc with Column as first
-        # argument
-        if type_id is names.COLUMN:
+        if isinstance(first_prop_arg, CallExpr):
+            type_id = names.type_id_for_callee(first_prop_arg.callee)
+
+            # look for column_property() / deferred() etc with Column as first
+            # argument
+            if type_id is names.COLUMN:
+                return _infer_type_from_decl_column(
+                    api,
+                    stmt,
+                    node,
+                    left_hand_explicit_type,
+                    right_hand_expression=first_prop_arg,
+                )
+
+    if isinstance(stmt.rvalue, CallExpr):
+        type_id = names.type_id_for_callee(stmt.rvalue.callee)
+        # this is probably not strictly necessary as we have to use the left
+        # hand type for query expression in any case.  any other no-arg
+        # column prop objects would go here also
+        if type_id is names.QUERY_EXPRESSION:
             return _infer_type_from_decl_column(
                 api,
                 stmt,
                 node,
                 left_hand_explicit_type,
-                right_hand_expression=first_prop_arg,
             )
 
-    return _infer_type_from_left_hand_type_only(
+    return infer_type_from_left_hand_type_only(
         api, node, left_hand_explicit_type
     )
 
@@ -372,21 +424,24 @@ def _infer_type_from_decl_column(
         elif isinstance(column_arg, (StrExpr,)):
             # x = Column("name", String), go to next argument
             continue
+        elif isinstance(column_arg, (LambdaExpr,)):
+            # x = Column("name", String, default=lambda: uuid.uuid4())
+            # go to next argument
+            continue
         else:
             assert False
 
     if callee is None:
         return None
 
-    if isinstance(callee.node, TypeInfo) and names._mro_has_id(
+    if isinstance(callee.node, TypeInfo) and names.mro_has_id(
         callee.node.mro, names.TYPEENGINE
     ):
-        python_type_for_type = _extract_python_type_from_typeengine(
+        python_type_for_type = extract_python_type_from_typeengine(
             api, callee.node, type_args
         )
 
         if left_hand_explicit_type is not None:
-
             return _infer_type_from_left_and_inferred_right(
                 api, node, left_hand_explicit_type, python_type_for_type
             )
@@ -396,7 +451,7 @@ def _infer_type_from_decl_column(
     else:
         # it's not TypeEngine, it's typically implicitly typed
         # like ForeignKey.  we can't infer from the right side.
-        return _infer_type_from_left_hand_type_only(
+        return infer_type_from_left_hand_type_only(
             api, node, left_hand_explicit_type
         )
 
@@ -423,7 +478,7 @@ def _infer_type_from_left_and_inferred_right(
 
     if not is_subtype(left_hand_explicit_type, python_type_for_type):
         effective_type = api.named_type(
-            "__sa_Mapped", [orig_python_type_for_type]
+            names.NAMED_TYPE_SQLA_MAPPED, [orig_python_type_for_type]
         )
 
         msg = (
@@ -472,7 +527,7 @@ def _infer_collection_type_from_left_and_inferred_right(
     )
 
 
-def _infer_type_from_left_hand_type_only(
+def infer_type_from_left_hand_type_only(
     api: SemanticAnalyzerPluginInterface,
     node: Var,
     left_hand_explicit_type: Optional[ProperType],
@@ -492,21 +547,23 @@ def _infer_type_from_left_hand_type_only(
         )
         util.fail(api, msg.format(node.name), node)
 
-        return api.named_type("__sa_Mapped", [AnyType(TypeOfAny.special_form)])
+        return api.named_type(
+            names.NAMED_TYPE_SQLA_MAPPED, [AnyType(TypeOfAny.special_form)]
+        )
 
     else:
         # use type from the left hand side
         return left_hand_explicit_type
 
 
-def _extract_python_type_from_typeengine(
+def extract_python_type_from_typeengine(
     api: SemanticAnalyzerPluginInterface,
     node: TypeInfo,
     type_args: Sequence[Expression],
 ) -> ProperType:
     if node.fullname == "sqlalchemy.sql.sqltypes.Enum" and type_args:
         first_arg = type_args[0]
-        if isinstance(first_arg, NameExpr) and isinstance(
+        if isinstance(first_arg, RefExpr) and isinstance(
             first_arg.node, TypeInfo
         ):
             for base_ in first_arg.node.mro:
@@ -514,7 +571,7 @@ def _extract_python_type_from_typeengine(
                     return Instance(first_arg.node, [])
             # TODO: support other pep-435 types here
         else:
-            return api.named_type("__builtins__.str", [])
+            return api.named_type(names.NAMED_TYPE_BUILTINS_STR, [])
 
     assert node.has_base("sqlalchemy.sql.type_api.TypeEngine"), (
         "could not extract Python type from node: %s" % node
